/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_ktrace.c,v 1.7 2006/07/11 23:06:09 phargrov Exp $
 *
 *
 * Tracing output for checkpoint/restart kernel code.
 */

#include "cr_module.h"

#include "cr_ktrace.h"

#if CR_KERNEL_TRACING

unsigned int cr_ktrace_mask = CR_KTRACE_DEFAULT;
static char  cr_trace_buf[4096];
static CR_DEFINE_SPINLOCK(cr_trace_lock);

void
cr_print_trace(const char *file, int line, const char *func, const char * fmt, ...)
{
	va_list args;
	const char *p;

	spin_lock(&cr_trace_lock);

	va_start(args, fmt);
	vsprintf(cr_trace_buf, fmt, args);
	va_end(args);

	/* simple basename implementation: */
        for (p = file; *p != '\0'; ++p) {
	    if (*p == '/') {
		file = p + 1;
	    }
	}

	printk(KERN_INFO "%s <%s:%d>, pid %d: %s\n", func, file, line,
	       current->pid, cr_trace_buf);

	spin_unlock(&cr_trace_lock);
}

#endif 
