/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_module.h,v 1.193.10.3 2007/12/11 20:17:11 phargrov Exp $
 */

#ifndef _CR_MODULE_H
#define _CR_MODULE_H        1

#include "blcr_config.h"	// Configuration

#include <linux/module.h>
#include <linux/autoconf.h>

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/list.h>
#include <linux/wait.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/file.h>
#include <linux/mount.h>
#include <linux/slab.h>
#include <linux/highmem.h>
#include <linux/spinlock.h>
#include <linux/vmalloc.h>
#include <asm/atomic.h>
#if HAVE_LINUX_NAMEI_H
  #include <linux/namei.h>
#endif
#if HAVE_LINUX_SYSCALLS_H
  #include <linux/syscalls.h>           /* for sys_close, etc. */
#endif
#if HAVE_LINUX_SYSCALL_H
  #include <linux/syscall.h>           /* for sys_close, etc. */
#endif
#if HAVE_LINUX_RCUPDATE_H
  #include <linux/rcupdate.h>
#endif
#if HAVE_LINUX_HASH_H
  #include <linux/hash.h>
#endif
#if defined(CONFIG_COMPAT)
  #include <linux/compat.h>
  /* This mapping works only because we only pass pointers and longs */
  #define CR_IOCTL32_MAP(x)	((((x)&0xff0000) == 0x80000) ? (((x)&~0xff0000)|0x40000) : (x))
  #define CR_IOCTL32_UNMAP(x)	((((x)&0xff0000) == 0x40000) ? (((x)&~0xff0000)|0x80000) : (x))
#endif

#include <blcr_common.h>	// For the stuff common to user space
#include <blcr_imports.h>	// Prototypes for non-exported and/or non-prototyped kernel symbols

#include "cr_arch.h"		// Architecture-specific bits
#include "cr_barrier.h"		// Code for kernel barriers.
#include "cr_ktrace.h"		// Code for trace messages
#include "vmadump.h"

#ifndef TASK_COMM_LEN
  #define TASK_COMM_LEN 16
#endif

// cr_objectmap_t is an opaque type
struct cr_objectmap_s;
typedef struct cr_objectmap_s *cr_objectmap_t;

// Foward type decls:
struct cr_shanon_desc;

// Source/Destination location info for a checkpoint
typedef	struct cr_location_s {
	// file pointer (NULL iff location is a directory)
	struct file *		filp;
	struct semaphore	mutex;

	// fs struct pointer (NULL iff location is not a directory)
	struct fs_struct *	fs;
} cr_location_t;

// MOD_{INC,DEC}_USE_COUNT are deprecated in 2.6.x
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0))
  #define CR_MODULE_GET()	try_module_get(THIS_MODULE)
  #define CR_MODULE_PUT()	module_put(THIS_MODULE)
  #undef MOD_INC_USE_COUNT
  #undef MOD_DEC_USE_COUNT
  #define MOD_INC_USE_COUNT	%%%%ERROR%%%% use CR_MODULE_GET()
  #define MOD_DEC_USE_COUNT	%%%%ERROR%%%% use CR_MODULE_PUT()
#else
  #define CR_MODULE_GET()	(MOD_INC_USE_COUNT, 1)
  #define CR_MODULE_PUT()	MOD_DEC_USE_COUNT
#endif

// Implement the 'struct delayed_work' in terms of the older 'struct work_struct' if needed
// Implement the 'struct work_struct' in terms of the older 'struct tq_struct' if needed
#if HAVE_STRUCT_DELAYED_WORK
  #if HAVE_LINUX_WORKQUEUE_H
    #include <linux/workqueue.h>
  #endif
  #define CR_INIT_WORK(_work, _task, _data, _func) \
	INIT_DELAYED_WORK((_work), (_task))
  typedef struct delayed_work cr_delayed_work_t;
#elif HAVE_STRUCT_WORK_STRUCT
  #if HAVE_LINUX_WORKQUEUE_H
    #include <linux/workqueue.h>
  #endif
  #define CR_INIT_WORK(_work, _task, _data, _func) \
	INIT_WORK((_work), (_task), (_data))
  typedef struct work_struct cr_delayed_work_t;
#elif HAVE_STRUCT_TQ_STRUCT
  typedef struct {
      struct tq_struct	task;
      struct timer_list	timer;
  } cr_delayed_work_t;
  #if HAVE_INIT_TQUEUE
    #ifndef INIT_TQUEUE
        #define INIT_TQUEUE(_tq, _func, _data) do {	\
	    INIT_LIST_HEAD(&(_tq)->list);		\
	    (_tq)->sync = 0;				\
	    (_tq)->routine = (_func);    		\
	    (_tq)->data = (_data);             		\
	  } while (0)
    #endif
    #define CR_INIT_WORK(_work, _task, _data, _func)  do {	\
	INIT_TQUEUE(&((_work)->task), (_task), (_data));	\
        init_timer(&((_work)->timer));				\
        (_work)->timer.function = (_func);			\
        (_work)->timer.data = (unsigned long)&(_work)->task;	\
      } while (0);
  #else
    #define CR_INIT_WORK(_work, _task, _data, _func)  do {	\
        INIT_LIST_HEAD(&((_work)->task.list));			\
        (_work)->task.sync = 0;					\
        (_work)->task.routine = &watchdog_task;			\
        (_work)->task.data = req;				\
        init_timer(&((_work)->timer));				\
        (_work)->timer.function = (_func);			\
        (_work)->timer.data = (unsigned long)&((_work)->task);	\
      } while (0);
  #endif
#else
  #error
#endif

#ifdef CAP_KILL
  #define cr_capable(X) capable(X)
#else
  #define cr_capable(X) suser()
#endif

#if HAVE_2_ARG_PERMISSION
  #define cr_permission(I,M,N)	permission((I),(M))
#elif HAVE_3_ARG_PERMISSION
  #define cr_permission(I,M,N)	permission((I),(M),(N))
#else
  #error
#endif

#if HAVE_TASK_RLIM
  #define CR_RLIM(task)	(task)->rlim
#elif HAVE_SIGNAL_RLIM
  #define CR_RLIM(task)	(task)->signal->rlim
#else
  #error
#endif

#ifndef rcu_assign_pointer
  #define rcu_assign_pointer(A,B)	((A) = (B))
#endif
#ifndef rcu_read_lock
  #define rcu_read_lock()		do {} while (0)
  #define rcu_read_unlock()		do {} while (0)
#endif

#if HAVE_STRUCT_FDTABLE
  typedef struct fdtable	cr_fdtable_t;
  #define cr_fdtable(files)	files_fdtable(files)
#else
  typedef struct files_struct	cr_fdtable_t;
  #define cr_fdtable(files)	(files)
#endif

#ifndef thread_group_leader
  #define thread_group_leader(p) ((p)->pid == (p)->tgid)
#endif

#if HAVE_I_SIZE_READ
  #define cr_i_size_read i_size_read
#else
  static __inline__ loff_t cr_i_size_read(const struct inode *inode) {
    return inode->i_size;
  }
#endif

#if HAVE_2_4_SIGNAL_STRUCT
  #define CR_SIGNAL_LOCK(_task)		spin_lock_irq(&(_task)->sigmask_lock)
  #define CR_SIGNAL_UNLOCK(_task)	spin_unlock_irq(&(_task)->sigmask_lock)
  #define CR_SIGACTION(_task,_num)	((_task)->sig->action[(_num)-1])
#elif HAVE_2_6_SIGNAL_STRUCT
  #define CR_SIGNAL_LOCK(_task)		spin_lock_irq(&(_task)->sighand->siglock)
  #define CR_SIGNAL_UNLOCK(_task)	spin_unlock_irq(&(_task)->sighand->siglock)
  #define CR_SIGACTION(_task,_num)	((_task)->sighand->action[(_num)-1])
#else
  #error "No signal struct?"
#endif
#define CR_SIGNAL_HAND(_task,_num)	(CR_SIGACTION((_task),(_num)).sa.sa_handler)

// Kernel-side tracking of a checkpoint request
//
// XXX: this needs a lot of stuff added as we progress:
// It will also keep such things as the count of tasks in the
// checkpoint, reference counts for resources, etc.
//
// XXX: the naming between user and kernel could be improved.
typedef struct {
	struct list_head	list;
	const struct mm_struct	*mm;
	struct cr_chkpt_req_s	*req;

	struct list_head	tasks;		// Tasks in this proc_req (protected by req->lock) */
	int			thread_count;
        struct file            *file;		// per-process context file (may equal master)
	struct semaphore        serial_mutex;   // globally order steps in req
	cr_barrier_t		phase_barrier;		// between phases 1 & 2
	cr_barrier_t		predump_barrier;	// at start of dump
	cr_barrier_t		vmadump_barrier;	// at end of vmadump
	int			omit;
	struct k_sigaction	saved_sa;	// Space to save user-registered sigaction.
	struct k_sigaction	forced_sa;	// Space to save blcr-registered sigaction.

	cr_bool_t               have_leader;
	cr_bool_t               done_init;
	cr_bool_t               done_header;
	cr_bool_t               done_linkage;
	cr_bool_t               done_creds;
	cr_bool_t               done_fs;
	cr_bool_t               done_shanon_maps;
	cr_bool_t               done_files;
	cr_bool_t               done_shanon_data;
	cr_bool_t               done_fini;

	/* To ensure req->signal (if non-zero) is delivered correctly */
	cr_barrier_t		presig_barrier;
	cr_barrier_t		postsig_barrier;
	cr_bool_t               done_sig;

	/* For vmadump thread management */
	wait_queue_head_t	wait;
	cr_bool_t               done_leader;

	/* For shared-anonymous mapping tracking */
	int                     shanon_cnt;
	struct cr_shanon_desc   *shanon_tbl;

	cr_bool_t		duplicate_flag;
} cr_chkpt_proc_req_t;
typedef struct cr_chkpt_req_s {
	pid_t			requester;	// who requested the checkpoint
	pid_t			target;		// who is to be checkpointed
	atomic_t		ref_count;	// reference count
	atomic_t		completed;	// count of successful completions
	rwlock_t		lock;		// spinlock for task_list & wait
	wait_queue_head_t	wait;		// place for requester to wait
	struct list_head	tasks;		// list of target cr_task_t's
	struct list_head	procs;		// list of target cr_chkpt_proc_req_t's
	cr_location_t		dest;		// checkpoint destination
	struct semaphore        serial_mutex;   // mutex for i/o serialization
        cr_bool_t               done_header;
        cr_bool_t               done_trailer;
	unsigned int		flags;		// flags supplied by requestor
	int			result;		// value returned by REAP
	int			die;		// flag indicating we must die
	int			has_expiration;	// 'expiration' is used
	unsigned long		expiration;	// limit on time to chkpt
	cr_delayed_work_t	watchdog;	// Watchdog task and timer
        cr_format_t             dump_format;    // Format to use for this dump
	int			signal;		// sent after CHECKPOINT
	cr_scope_t              checkpoint_scope;
	cr_objectmap_t          map;
	cr_barrier_t            preshared_barrier; // before files & shanon_data
	cr_barrier_t		postdump_barrier; // at end of dump
} cr_chkpt_req_t;

typedef volatile enum {
	CR_RSTRT_STATE_REQUESTER,	// Requester needs to fork() child to read context
	CR_RSTRT_STATE_CHILD,		// A child is reading the context file
	CR_RSTRT_STATE_DONE		// Finished, possibly in error
} cr_rstrt_state_t;

typedef struct {
	struct list_head	list;
	const struct mm_struct	*mm;
	struct cr_rstrt_req_s	*req;

	cr_bool_t               have_leader;
        cr_bool_t               done_fs;
	cr_bool_t               done_shanon_maps;
        cr_bool_t               done_files;
	cr_bool_t               done_shanon_data;
	cr_bool_t               done_hide_cr_fds;
	cr_bool_t               done_close_cr_fds;
	cr_bool_t               done_creds;
	cr_bool_t               done_linkage;
	cr_bool_t               done_abort;
	int                     clone_flags;
	int	                clones_needed;
	struct cr_context_tasklinkage *linkage;
	int                     linkage_size;
	int                     thread_count;
	/* barriers for cr_restore_linkage */
	atomic_t		linkage_counter;
	cr_barrier_t		linkage_barrier;
	struct semaphore        serial_mutex;
        struct file            *file;		// per-process context file (may equal master)
	atomic_t		final_counter;
	/* For vmadump thread management and inter-thread communication */
	cr_barrier_t		vmadump_barrier;
	wait_queue_head_t	wait;
	cr_bool_t               done_leader;
	char			comm[TASK_COMM_LEN];
	int			thaw_error;
	VMAD_VDSO_DATA

	/* To ensure req->signal (if non-zero) is delivered correctly */
	cr_barrier_t		postsig_barrier;
	cr_bool_t               done_sig;

	/* For shared-anonymous mapping tracking */
	int                     shanon_cnt;
	struct cr_shanon_desc   *shanon_tbl;
} cr_rstrt_proc_req_t;
typedef struct cr_rstrt_req_s {
	pid_t			requester;	// who requested the restart
        cr_location_t           src;
	atomic_t                ref_count;
	struct task_struct     *cr_restart_task;
	struct file            *cr_restart_stdin;
	struct file            *cr_restart_stdout;
	cr_rstrt_state_t        state;
	wait_queue_head_t	wait;		// place for requester to wait for state changes
	int			result;		// value returned by REAP
	int			die;		// indicates spawned threads must die
	cr_objectmap_t          map;		// map from old object addrs to new
	cr_bool_t               need_children;	// XXX: needs replacement for multi-process case
        struct file            *file0;		// "master" context file
	cr_barrier_t		barrier;
	rwlock_t		procs_lock;	// protects procs list
	struct list_head	procs;		// list of target cr_rstrt_proc_req_t's
	cr_scope_t		scope;
	int			signal;
} cr_rstrt_req_t;

typedef enum {
	CR_NO_PHASE = 0,
	CR_PHASE1,
	CR_PHASE2
} cr_phase_t;

// For forming a list of tasks.
typedef struct cr_task_s {
	struct list_head	task_list;	// All tasks
	struct list_head	req_list;	// Tasks in same req
	struct list_head	proc_req_list;	// Tasks in same proc_req
	atomic_t		ref_count;
	struct task_struct	*task;
	cr_chkpt_req_t		*chkpt_req;
	cr_chkpt_proc_req_t	*chkpt_proc_req;
	cr_rstrt_req_t		*rstrt_req;
	cr_rstrt_proc_req_t	*rstrt_proc_req;
	cr_phase_t		phase;
	__sighandler_t		handler;
	struct k_sigaction	handler_sa;
	struct file		*filp;		// associated w/ handler (if any)
	int			stopped;
} cr_task_t;

// Private data attached to an instance of the file
typedef struct cr_pdata_s {
	cr_chkpt_req_t		*chkpt_req;
	// cr_chkpt_opt_t	*chkpt_opt;	NOT YET DEFINED
	cr_rstrt_req_t	        *rstrt_req;
} cr_pdata_t;

#if !HAVE_TIMEVAL_TO_JIFFIES
	// Basic adaptation of timespec_to_jiffies from include/linux/time.h
	static __inline__ unsigned long
	timeval_to_jiffies(struct timeval *value)
	{
		unsigned long sec = value->tv_sec;
		long usec = value->tv_usec;

		if (sec >= (MAX_JIFFY_OFFSET / HZ))
			return MAX_JIFFY_OFFSET;
		usec += 1000000L / HZ - 1;
		usec /= 1000000L / HZ;
		return HZ * sec + usec;
	}
#endif

#if !HAVE_JIFFIES_TO_TIMEVAL
	// Basic adaptation of jiffies_to_timeval from include/linux/time.h
	static __inline__ void
	jiffies_to_timeval(unsigned long jiffies, struct timeval *value)
	{
		value->tv_usec = (jiffies % HZ) * (1000000L / HZ);
		value->tv_sec = jiffies / HZ;
	}
#endif
	
#ifndef list_for_each_safe
// list_for_each_safe: A list iterator allowing for deletions
// First appears in 2.4.10 kernel, from which we've taken the following:
  #define list_for_each_safe(pos, n, head) \
    for (pos = (head)->next, n = pos->next;\
         pos != (head);\
         pos = n, n = pos->next)
#endif

// Task accessor macros (must be lvalues if/when we ever allow restoring them)
#if HAVE_2_6_20_TASK_IDS
  #define cr_task_pgrp(_t)	((_t)->signal->pgrp)
  #define cr_task_tty_old_pgrp(_t)	((_t)->signal->tty_old_pgrp)
  #define cr_task_session(_t)	process_session(_t)
  #define cr_task_leader(_t)	((_t)->signal->leader)
  #define cr_task_tty(_t)	((_t)->signal->tty)
#elif HAVE_2_6_6_TASK_IDS
  #define cr_task_pgrp(_t)	((_t)->signal->pgrp)
  #define cr_task_tty_old_pgrp(_t)	((_t)->signal->tty_old_pgrp)
  #define cr_task_session(_t)	((_t)->signal->session)
  #define cr_task_leader(_t)	((_t)->signal->leader)
  #define cr_task_tty(_t)	((_t)->signal->tty)
#elif HAVE_2_6_0_TASK_IDS
  #define cr_task_pgrp(_t)	((_t)->__pgrp)
  #define cr_task_tty_old_pgrp(_t)	((_t)->tty_old_pgrp)
  #define cr_task_session(_t)	((_t)->session)
  #define cr_task_leader(_t)	((_t)->leader)
  #define cr_task_tty(_t)	((_t)->tty)
#elif HAVE_2_4_TASK_IDS
  #define cr_task_pgrp(_t)	((_t)->pgrp)
  #define cr_task_tty_old_pgrp(_t)	((_t)->tty_old_pgrp)
  #define cr_task_session(_t)	((_t)->session)
  #define cr_task_leader(_t)	((_t)->leader)
  #define cr_task_tty(_t)	((_t)->tty)
#else
  #error
#endif

// Process table iterators
#if HAVE_DO_EACH_PID_TASK && !HAVE_DO_EACH_TASK_PID
  #define do_each_task_pid(ID, TYPE, T)          \
    do {                                         \
      struct pid *the_pid = find_pid(ID);        \
      do_each_pid_task(the_pid, (TYPE), (T))
  #define while_each_task_pid(ID, TYPE, T)       \
      while_each_pid_task(the_pid, (TYPE), (T)); \
    } while (0)
  #undef HAVE_DO_EACH_TASK_PID
  #define HAVE_DO_EACH_TASK_PID 1
#endif
#if HAVE_DO_EACH_TASK_PID
  #define CR_DO_EACH_TASK_PGID(ID, T) \
    do_each_task_pid((ID), PIDTYPE_PGID, (T))
  #define CR_WHILE_EACH_TASK_PGID(ID, T) \
    while_each_task_pid((ID), PIDTYPE_PGID, (T))

  #define CR_DO_EACH_TASK_SID(ID, T) \
    do_each_task_pid((ID), PIDTYPE_SID, (T))
  #define CR_WHILE_EACH_TASK_SID(ID, T) \
    while_each_task_pid((ID), PIDTYPE_SID, (T))

  #if HAVE_PIDTYPE_TGID
    #define CR_DO_EACH_TASK_TGID(ID, T) \
      do_each_task_pid((ID), PIDTYPE_TGID, (T))
    #define CR_WHILE_EACH_TASK_TGID(ID, T) \
      while_each_task_pid((ID), PIDTYPE_TGID, (T))
  #elif HAVE_TASK_THREAD_GROUP
    #define CR_DO_EACH_TASK_TGID(ID, T) \
      { \
	struct task_struct *_leader = find_task_by_pid(ID); \
	if (_leader) { \
	  struct list_head *_pos; \
	  /* "extra" iteration for thread group leader (the list head): */ \
	    _pos = &(_leader->thread_group); (T) = _leader; goto _label; \
	  list_for_each(_pos, &(_leader->thread_group)) { \
	    (T) = list_entry(_pos, struct task_struct, thread_group); \
	    _label:
    #define CR_WHILE_EACH_TASK_TGID(ID, T) \
	  } \
	} \
      }
  #else
    #error "CR_DO_EACH_TASK_TGID not implemented"
  #endif

  #define CR_DO_EACH_TASK_PROC(P, T) \
    if ((P)->mm) { \
      pid_t _id = cr_task_pgrp(P); \
      CR_DO_EACH_TASK_PGID(_id, (T)) \
        if ((T)->mm == (P)->mm)
  #define CR_WHILE_EACH_TASK_PROC(ID, T) \
      while_each_task_pid(_id, PIDTYPE_PGID, (T)); \
    }
#elif HAVE_FOR_EACH_TASK_PID
  #define CR_DO_EACH_TASK_PGID(ID, T) \
    do { \
      struct list_head *_l; \
      struct pid *_p; \
      for_each_task_pid((ID), PIDTYPE_PGID, (T), _l, _p)
  #define CR_WHILE_EACH_TASK_PGID(ID, T) \
    } while(0)

  #define CR_DO_EACH_TASK_SID(ID, T) \
    do { \
      struct list_head *_l; \
      struct pid *_p; \
      for_each_task_pid((ID), PIDTYPE_SID, (T), _l, _p)
  #define CR_WHILE_EACH_TASK_SID(ID, T) \
    } while(0)

  #define CR_DO_EACH_TASK_TGID(ID, T) \
    do { \
      struct list_head *_l; \
      struct pid *_p; \
      for_each_task_pid((ID), PIDTYPE_TGID, (T), _l, _p)
  #define CR_WHILE_EACH_TASK_TGID(ID, T) \
    } while(0)

  #define CR_DO_EACH_TASK_PROC(P, T) \
    if ((P)->mm) { \
      pid_t _id = cr_task_pgrp(P); \
      CR_DO_EACH_TASK_PGID(_id, (T)) { \
        if ((T)->mm == (P)->mm) {
  #define CR_WHILE_EACH_TASK_PROC(P, T) \
        } \
      } CR_WHILE_EACH_TASK_PGID(_id, (T)); \
    }

#elif HAVE_FOR_EACH_TASK
  #define CR_DO_EACH_TASK_PGID(ID, T) \
    for_each_task((T)) if (cr_task_pgrp(T) == (ID)) 
  #define CR_WHILE_EACH_TASK_PGID(ID, T)

  #define CR_DO_EACH_TASK_SID(ID, T) \
    for_each_task((T)) if (cr_task_session(T) == (ID)) 
  #define CR_WHILE_EACH_TASK_SID(ID, T)

  #define CR_DO_EACH_TASK_TGID(ID, T) \
    %%%ERROR%%% Use of TGID field with for_each_task
  #define CR_WHILE_EACH_TASK_TGID(ID, T) \
    %%%ERROR%%% Use of TGID field with for_each_task

  #define CR_DO_EACH_TASK_PROC(P, T) \
    if ((P)->mm) \
      for_each_task((T)) \
        if ((T)->mm == (P)->mm) 
  #define CR_WHILE_EACH_TASK_PROC(P, T)
#else
  #error No tasklist iterators available
#endif

#if HAVE_2_4_TASK_LINKAGE
  #define CR_DO_EACH_CHILD(C, T) \
    for ((C) = task->p_cptr; (C); (C) = (C)->p_osptr)
  #define CR_WHILE_EACH_CHILD(C, T)
#elif HAVE_2_6_TASK_LINKAGE
  #define CR_DO_EACH_CHILD(C, T) \
    do { \
      struct list_head *_elem; \
      list_for_each(_elem, &(T)->children) { \
        (C) = list_entry(_elem, struct task_struct, sibling);
  #define CR_WHILE_EACH_CHILD(C, T) \
      } \
    } while (0)
#endif

#if HAVE_FIND_TASK_BY_PID_TYPE
  #define cr_have_pid(T,P) (find_task_by_pid_type((T),(P)) != NULL)
#else
  #define cr_have_pid(T,P) (find_pid((T),(P)) != NULL)
#endif

/* What is the opposite of "get_task_struct" ? */
#if HAVE_FREE_TASK_STRUCT
  #define put_task_struct free_task_struct
#elif HAVE_PUT_TASK_STRUCT
  /* will CR_IMPORT_KSYM(__put_task_struct); */
#else
  #error
#endif

/* How do we manipulate the file_lock */
#if HAVE_RW_FILE_LOCK
  #define cr_file_lock_read		read_lock
  #define cr_file_lock_unread		read_unlock
  #define cr_file_lock_write		write_lock
  #define cr_file_lock_unwrite		write_unlock
#elif HAVE_SPINLOCK_FILE_LOCK
  #define cr_file_lock_read		spin_lock
  #define cr_file_lock_unread		spin_unlock
  #define cr_file_lock_write		spin_lock
  #define cr_file_lock_unwrite		spin_unlock
#else
  #error "Unknown file_lock type"
#endif 

/* How do we manipulate the lock on a inode */
#if HAVE_INODE_SEM
  #define cr_inode_lock(_i)			down(&(_i)->i_sem)
  #define cr_inode_lock_interruptible(_i)	down_interruptible(&(_i)->i_sem)
  #define cr_inode_unlock(_i)			up(&(_i)->i_sem)
#elif  HAVE_INODE_MUTEX
  #define cr_inode_lock(_i)			mutex_lock(&(_i)->i_mutex)
  #define cr_inode_lock_interruptible(_i)	mutex_lock_interruptible(&(_i)->i_mutex)
  #define cr_inode_unlock(_i)			mutex_unlock(&(_i)->i_mutex)
#else
  #error "Unknown inode lock type"
#endif 

#ifndef wait_event_interruptible_timeout
/* from 2.6.8 */
  #define __wait_event_interruptible_timeout(wq, condition, ret) \
    do {                                                         \
      DECLARE_WAITQUEUE(__wait, current);                        \
      add_wait_queue(&(wq), &__wait);                            \
      for (;;) {                                                 \
        set_current_state(TASK_INTERRUPTIBLE);                   \
        if (condition) break;                                    \
        if (!signal_pending(current)) {                          \
	  ret = schedule_timeout(ret);                           \
	  if (!ret) break;                                       \
	  continue;                                              \
        }                                                        \
        ret = -ERESTARTSYS;                                      \
        break;                                                   \
      }                                                          \
      current->state = TASK_RUNNING;                             \
      remove_wait_queue(&(wq), &__wait);                         \
    } while(0)
  #define wait_event_interruptible_timeout(wq, condition, timeout) \
    ({ long __ret = timeout;                                       \
       if (!(condition))                                           \
         __wait_event_interruptible_timeout(wq, condition, __ret); \
       __ret;                                                      \
    })
#endif

#if defined(EXIT_ZOMBIE)
  #define cri_task_dead(task)	 ((task)->exit_state & (EXIT_DEAD | EXIT_ZOMBIE))
  #define cri_task_zombie(task)	 ((task)->exit_state & EXIT_ZOMBIE)
#elif defined(TASK_DEAD)
  #define cri_task_dead(task)	 ((task)->state & (TASK_DEAD | TASK_ZOMBIE))
  #define cri_task_zombie(task)	 ((task)->state & TASK_ZOMBIE)
#else
  #define cri_task_dead(task)	 ((task)->state == TASK_ZOMBIE)
  #define cri_task_zombie(task)	 ((task)->state == TASK_ZOMBIE)
#endif

#ifdef DEFINE_SPINLOCK
  #define CR_DEFINE_SPINLOCK DEFINE_SPINLOCK
#else
  #define CR_DEFINE_SPINLOCK(_l) spinlock_t _l = SPIN_LOCK_UNLOCKED
#endif

/* Use path_lookup() if present */
#if HAVE_PATH_LOOKUP
  /* Macro or function (possibly unexported) */
  #define cr_path_lookup path_lookup
#else
  /* based on linux/fs/namei.c:path_lookup in later 2.4.x kernels */
  static __inline__ int cr_path_lookup(const char *path, unsigned flags, struct nameidata *nd) {
    return path_init(path, flags, nd) ? path_walk(path, nd) : 0;
  }
#endif

/* We use __putname() if we can, putname() otherwise */
#if !HAVE___PUTNAME
  #define __putname putname
#endif

// cr_proc.c
extern int cr_proc_init(void);
extern void cr_proc_cleanup(void);

// cr_fops.c
extern struct file_operations cr_ctrl_fops;
#if defined(CONFIG_COMPAT) && !defined(HAVE_COMPAT_IOCTL)
extern int cr_init_ioctl32(void);
extern int cr_cleanup_ioctl32(void);
#endif

// cr_dump_self.c
extern int cr_dump_self(struct file *filp, unsigned long flags);
extern int cr_cont_complete(struct file *filp);

// cr_chkpt_req.c
extern int cr_chkpt_req(struct file *filp, void *arg);
extern int cr_chkpt_done(struct file *filp, void *arg);
extern int cr_chkpt_reap(struct file *filp);
extern void cr_chkpt_task_complete(cr_task_t *cr_task);
extern int cr_chkpt_fwd(struct file *filp, void *arg);
extern void cr_chkpt_req_release(struct file *filp, cr_pdata_t *priv);
extern int cr_abort_checkpoint(struct file *filp, unsigned long flags);
extern int cr_chkpt_info(struct file *filp, struct cr_chkpt_info *uinfo);

// cr_async.c
extern int cr_suspend(struct file *filp, void *arg);
extern int cr_phase1_register(struct file *filp, void *arg);
extern void cr_phase1_release(struct file *filp, cr_pdata_t *priv);

// cr_sync.c
extern int cr_phase2_register(struct file *filp, void *arg);
extern void cr_phase2_release(struct file *filp, cr_pdata_t *priv);

// cr_task.c
extern rwlock_t cr_task_lock;
extern struct list_head cr_task_list;
extern cr_task_t *__cr_task_get(struct task_struct *task, int create);
extern void __cr_task_put(cr_task_t *cr_task);
extern cr_task_t *cr_task_get(struct task_struct *task);
extern void cr_task_put(cr_task_t *cr_task);

// cr_rstrt_req.c
extern int cr_rstrt_request_restart(struct file *filp, void *arg);
extern int cr_rstrt_done(struct file *filp, void *arg);
extern int cr_rstrt_reap(struct file *filp);
extern int cr_rstrt_child(struct file *filp);
extern int cr_rstrt_procs(struct file *filp, struct cr_procs_tbl *tbl_ptr);
extern int cr_rstrt_complete(struct file *filp);
extern int cr_rstrt_src(struct file *filp, char *arg);
extern int cr_rstrt_req_release(struct file *filp, cr_pdata_t *priv);

// cr_dest_file.c
extern int cr_dest_init(cr_location_t *dest, int fd, struct file *from);
extern void cr_dest_free(cr_location_t *dest);
extern struct file *cr_dest_get(cr_location_t *dest, int *shared);
extern void cr_dest_put(cr_location_t *dest, struct file *filp);

// cr_trigger.c
extern int __cr_trigger_phase1(cr_chkpt_proc_req_t *proc_req);
extern int __cr_trigger_phase1_only(cr_chkpt_proc_req_t *req);
extern int cr_trigger_phase1(cr_chkpt_req_t *req);
extern int __cr_trigger_phase2(cr_chkpt_proc_req_t *proc_req);
extern int cr_trigger_phase2(cr_chkpt_req_t *req, cr_chkpt_proc_req_t *proc_req);

// cr_io.c
extern ssize_t cr_uread(struct file * file, void *buf, size_t count);
extern ssize_t cr_uwrite(struct file * file, const void *buf, size_t count);
extern ssize_t cr_kread(struct file * file, void *buf, size_t count);
extern ssize_t cr_kwrite(struct file * file, const void *buf, size_t count);
extern int cr_skip(struct file *filp, loff_t len);
extern int cr_fgets(char *buf, int size, struct file *filp);
extern int cr_fputs(const char *buf, struct file *filp);
extern int cr_save_pathname(struct file *cr_filp, struct dentry *dentry, struct vfsmount *mnt, char *buf, int size);
extern int cr_save_filename(struct file *cr_filp, struct file *filp, char *buf, int size);
extern char *cr_getname(struct file *filp, int null_ok);
extern char *cr_location2path(cr_location_t *loc, char *buf, int size);
extern struct file * cr_dentry_open(struct dentry *dentry, struct vfsmount *mnt, int flags);
extern struct dentry *cr_mknod(struct nameidata *nd, const char *name, int mode, unsigned long unlinked_id);
extern struct file *cr_filp_mknod(const char *name, int mode, int flags, unsigned long unlinked_id);
extern int cr_filp_chmod(struct file *filp, mode_t mode);
extern struct file *cr_mkunlinked(struct file *cr_filp, const char *name, int mode, int flags, loff_t size, unsigned long unlinked_id);
extern loff_t cr_copyfile(struct file *dst_filp, loff_t *dst_ppos, struct file *src_filp, loff_t *src_ppos, loff_t count);
extern struct dentry *cr_link(struct dentry *old_dentry, struct vfsmount *mnt, const char *name);
extern struct file *cr_filp_reopen(const struct file *orig_filp, int new_flags);
extern int cr_fd_claim(int fd);

// cr_objects.c
extern cr_objectmap_t cr_alloc_objectmap(void);
extern void cr_release_objectmap(cr_objectmap_t);
extern int cr_find_object(cr_objectmap_t, void *, void **);
extern int cr_insert_object(cr_objectmap_t, void *, void *);
extern int cr_remove_object(cr_objectmap_t, void *);

#ifdef CONFIG_COMPAT
// cr_compat.c
extern long cr_compat_ctrl_ioctl(struct file *, unsigned, unsigned long);
#endif

// cr_vmadump.c
extern loff_t cr_freeze_threads(cr_chkpt_proc_req_t *proc_req, int flags,
				int i_am_leader);
extern long cr_thaw_threads(cr_rstrt_proc_req_t *proc_req, int flags,
                            int i_am_leader);

// cr_shanon.c
extern long cr_save_shanon_maps(cr_chkpt_proc_req_t *proc_req);
extern loff_t cr_save_shanon_data(cr_chkpt_proc_req_t *proc_req);
extern long cr_load_shanon_maps(cr_rstrt_proc_req_t *proc_req);
extern long cr_load_shanon_data(cr_rstrt_proc_req_t *proc_req);

#endif
