/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2007, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: blcr_vmadump.h,v 1.7 2007/08/24 21:10:19 phargrov Exp $
 *
 * This file provides declarations for BLCR-specific VMADump extensions.
 */
#ifndef _BLCR_VMADUMP_H
#define _BLCR_VMADUMP_H
 
#ifndef EXPORT_SYMBOL_GPL
  #define EXPORT_SYMBOL_GPL EXPORT_SYMBOL
#endif

/* Not maintaining the binfmt code */
#undef HAVE_BINFMT_VMADUMP

/* Additional flags */
#define VMAD_DUMP_REGSONLY 0x1000	/* Only thread-specific info */
#define VMAD_DUMP_NOSHANON 0x2000	/* Omit shared anonymous mappings */

#ifdef __KERNEL__
#include "blcr_config.h"
#include "blcr_imports.h"

/* Here because defn must be common to vmadump and cr_vmadump.c: */
static inline int vmad_dentry_unlinked(struct dentry *dentry) {
  return ((!IS_ROOT(dentry) && d_unhashed(dentry)) ||
          (dentry->d_inode->i_nlink == 0) ||
	  (dentry->d_flags & DCACHE_NFSFS_RENAMED));
}

/* Overload the namelen flag to store the VDSO */
#define VMAD_NAMELEN_VDSO (PAGE_SIZE+1)

/* Note care taken w/ map_vsyscall():
 * + FC5/x86 declares the symbol but never defines it.
 * + x86_64 kernels have a symbol by the same name, but w/ a different purpose
 * So, we look for both CR_KCODE_map_vsyscall and HAVE_MAP_VSYSCALL
 * Later we can just look for (VMAD_HAVE_VDSO && HAVE_MAP_VSYSCALL).
 */
#if defined(ARCH_HAS_SETUP_ADDITIONAL_PAGES) || \
    (defined(CR_KCODE_map_vsyscall) && HAVE_MAP_VSYSCALL) || \
	((VMAD_ARCH == VMAD_ARCH_x86_64)) && (defined(CR_KCODE_syscall32_setup_pages))
  #define VMAD_HAVE_VDSO 1
#else
  #define VMAD_HAVE_VDSO 0
#endif
#if VMAD_HAVE_VDSO
  #if (VMAD_ARCH == VMAD_ARCH_ppc64) && (HAVE_THREAD_VDSO_BASE)
    #define vmad_vdso_base(task) ((task)->thread.vdso_base)
    #define vmad_vdso_move(_old_addr, _new_addr) \
                (current->thread.vdso_base = _new_addr)
    #define vmad_vdso_save(_req) \
                ((_req)->vdso_base = current->thread.vdso_base)
    #define vmad_vdso_load(_req) \
                (current->thread.vdso_base = (_req)->vdso_base)
    #define VMAD_VDSO_DATA      unsigned long vdso_base;
  #elif (VMAD_ARCH == VMAD_ARCH_ppc64) && (HAVE_MM_CONTEXT_VDSO_BASE)
    #define vmad_vdso_base(task) ((task)->mm->context.vdso_base)
    #define vmad_vdso_move(_old_addr, _new_addr) \
                (current->mm->context.vdso_base = _new_addr)
    #define vmad_vdso_save(_req)
    #define vmad_vdso_load(_req)
    #define VMAD_VDSO_DATA      unsigned long vdso_base;
  #elif (VMAD_ARCH == VMAD_ARCH_i386)
    #if HAVE_MM_CONTEXT_VDSO
      #define vmad_vdso_base(task) ((unsigned long)((task)->mm->context.vdso))
      #define vmad_vdso_move_base(_old_addr,_new_addr) \
		current->mm->context.vdso = (void *)_new_addr
    #elif HAVE_VSYSCALL_BASE
      #define vmad_vdso_base(task) ((unsigned long)VSYSCALL_BASE)
      #define vmad_vdso_move_base(_old_addr,_new_addr) do {} while(0)
    #else
      #error "No support yet for VDSO on your kernel - please report as a BLCR bug"
    #endif
    #if HAVE_THREAD_INFO_SYSENTER_RETURN
      #define vmad_vdso_move_sysenter(_old_addr, _new_addr) do {                           \
		current_thread_info()->sysenter_return =                                   \
			(void *)((_new_addr - _old_addr) +                                 \
				 (unsigned long)(current_thread_info()->sysenter_return)); \
    	  } while (0)
      #define vmad_vdso_save(_req) \
		((_req)->sysenter_return = current_thread_info()->sysenter_return)
      #define vmad_vdso_load(_req) \
		(current_thread_info()->sysenter_return = (_req)->sysenter_return)
      #define VMAD_VDSO_DATA	void * sysenter_return;
    #else
      #define vmad_vdso_move_sysenter(_old_addr, _new_addr) do {} while(0)
      #define vmad_vdso_save(_req) do {} while(0)
      #define vmad_vdso_load(_req) do {} while(0)
      #define VMAD_VDSO_DATA
    #endif
    #define vmad_vdso_move(_old_addr, _new_addr) do {         \
		vmad_vdso_move_base(_old_addr,_new_addr);     \
		vmad_vdso_move_sysenter(_old_addr,_new_addr); \
    	  } while (0)
  #elif ((VMAD_ARCH == VMAD_ARCH_x86_64)) && (defined(CR_KCODE_syscall32_setup_pages))
    #if HAVE_ASM_VSYSCALL32_H
      #include <asm/vsyscall32.h>
    #endif
    #define vmad_vdso_base(task) (test_thread_flag(TIF_IA32) ? ((unsigned long)VSYSCALL32_BASE) : 1UL)
    #define vmad_vdso_move(_old_addr,_new_addr) do {} while(0)
    #define vmad_vdso_save(_req)
    #define vmad_vdso_load(_req)
    #define VMAD_VDSO_DATA
  #else
    #error "No support yet for VDSO on your kernel - please report as a BLCR bug"
  #endif
  #define is_vdso_page(vma) ((vma)->vm_start == vmad_vdso_base(current))
#else
  #define is_vdso_page(vma) (0)
  #define VMAD_VDSO_DATA /* empty */
#endif

#if HAVE_RW_MMAP_SEM
  #define DOWN_READ_MMAP_SEM(mm)      down_read(&(mm)->mmap_sem)
  #define DOWN_WRITE_MMAP_SEM(mm)     down_write(&(mm)->mmap_sem)
  #define UP_READ_MMAP_SEM(mm)        up_read(&(mm)->mmap_sem)
  #define UP_WRITE_MMAP_SEM(mm)       up_write(&(mm)->mmap_sem)
#else
  #define DOWN_READ_MMAP_SEM(mm)      down(&(mm)->mmap_sem)
  #define DOWN_WRITE_MMAP_SEM(mm)     down(&(mm)->mmap_sem)
  #define UP_READ_MMAP_SEM(mm)        up(&(mm)->mmap_sem)
  #define UP_WRITE_MMAP_SEM(mm)       up(&(mm)->mmap_sem)
#endif

#endif /* defined(__KERNEL__) */

#endif
