/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2007, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_arch.h,v 1.3 2007/08/31 22:17:20 phargrov Exp $
 *
 * Experimental ARM support contributed by Anton V. Uzunov
 * <anton.uzunov@dsto.defence.gov.au> of the Australian Government
 * Department of Defence, Defence Science and Technology Organisation.
 *
 * ARM-specific questions should be directed to blcr-arm@hpcrd.lbl.gov.
 */

#ifndef _CR_ARCH_H
#define _CR_ARCH_H	1

// Catch-all file for misc. arch-specific bits

// macros for using the syscall instruction (SWI)
#if defined(__thumb__) || defined(__ARM_EABI__)
#  define CRI_SYS_REG(name)            register long __sysreg __asm__( "r7" ) = name;
#  define CRI_SYS_REG_LIST(regs...)    "r" (__sysreg) , regs
#  define CRI_SYS_REG_LIST_EABI_0()    "r" (__sysreg)
#  define CRI_syscall(name)            "swi \t 0"
#else
#  define CRI_SYS_REG(name)
#  define CRI_SYS_REG_LIST(regs...)    regs
#  define CRI_SYS_REG_LIST_EABI_0()
#  define CRI_syscall(name)            "swi \t" #name ""
#endif

#define cri_syscall_return(type, res, errno_p)				\
  do {									\
    if ((unsigned long) (res) >= (unsigned long) (-4096)) {		\
      if (errno_p != NULL) { *errno_p = -res; }		        	\
      errno = -(res);							\
      res = -1;						       		\
    }									\
    return( (type) (res) );						\
  } while (0)

#define cri_syscall0(type,name,nr)					\
  type name(int *errno_p) {						\
    CRI_SYS_REG( nr )							\
    register long __res_r0 __asm__( "r0" );				\
    long __res;								\
    __asm__ __volatile__ (						\
      CRI_syscall( nr )		     			   		\
  	: "=r" (__res_r0)						\
        : CRI_SYS_REG_LIST_EABI_0()					\
	: "memory" );							\
    __res = __res_r0;							\
    cri_syscall_return(type,__res,errno_p);				\
  }

#define cri_syscall1(type,name,nr,type1)				\
  type name(type1 arg1, int *errno_p) { 				\
    CRI_SYS_REG( nr )							\
    register long __r0 __asm__( "r0" ) = (long) arg1;			\
    register long __res_r0 __asm__( "r0" );				\
    long __res;					       			\
    __asm__ __volatile__ (						\
      CRI_syscall( nr )				       			\
  	: "=r" (__res_r0)						\
  	: CRI_SYS_REG_LIST( "0" (__r0) )				\
  	: "memory" );							\
    __res = __res_r0;							\
    cri_syscall_return(type,__res,errno_p);				\
  }

#define cri_syscall2(type,name,nr,type1,type2)				\
  type name(type1 arg1,type2 arg2, int *errno_p) {			\
    CRI_SYS_REG( nr )							\
    register long __r0 __asm__( "r0" ) = (long) arg1;			\
    register long __r1 __asm__( "r1" ) = (long) arg2;			\
    register long __res_r0 __asm__( "r0" );				\
    long __res;					       			\
    __asm__ __volatile__ (						\
      CRI_syscall( nr )				       			\
  	: "=r" (__res_r0)						\
  	: CRI_SYS_REG_LIST( "0" (__r0), "r" (__r1) )			\
  	: "memory" );							\
    __res = __res_r0;							\
    cri_syscall_return( type, __res, errno_p );	       			\
  }

#define cri_syscall3(type,name,nr,type1,type2,type3)			\
  type name(type1 arg1,type2 arg2,type3 arg3, int *errno_p) {		\
    CRI_SYS_REG( nr )							\
    register long __r0 __asm__( "r0" ) = (long) arg1;			\
    register long __r1 __asm__( "r1" ) = (long) arg2;			\
    register long __r2 __asm__( "r2" ) = (long) arg3;			\
    register long __res_r0 __asm__( "r0" );				\
    long __res;					       			\
    __asm__ __volatile__ (						\
      CRI_syscall( nr )				       			\
  	: "=r" (__res_r0)						\
  	: CRI_SYS_REG_LIST( "0" (__r0), "r" (__r1), "r" (__r2) )	\
  	: "memory" );							\
    __res = __res_r0;							\
    cri_syscall_return( type, __res, errno_p );	       			\
  }

#define cri_syscall4(type,name,nr,type1,type2,type3,type4)		\
  type name(type1 arg1, type2 arg2, type3 arg3, type4 arg4, int *errno_p) { \
    CRI_SYS_REG( nr )							\
    register long __r0 __asm__( "r0" ) = (long) arg1;			\
    register long __r1 __asm__( "r1" ) = (long) arg2;			\
    register long __r2 __asm__( "r2" ) = (long) arg3;			\
    register long __r3 __asm__( "r3" ) = (long) arg4;			\
    register long __res_r0 __asm__( "r0" );				\
    long __res;					       			\
    __asm__ __volatile__ (						\
      CRI_syscall( nr )				       			\
  	: "=r" (__res_r0)						\
  	: CRI_SYS_REG_LIST( "0" (__r0), "r" (__r1), "r" (__r2), "r" (__r3) ) \
  	: "memory" );							\
    __res = __res_r0;							\
    cri_syscall_return( type, __res, errno_p );	       			\
  }

#define cri_syscall5(type,name,nr,type1,type2,type3,type4,type5)	\
  type name(type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5, int *errno_p) { \
    CRI_SYS_REG( nr )							\
    register long __r0 __asm__( "r0" ) = (long) arg1;			\
    register long __r1 __asm__( "r1" ) = (long) arg2;			\
    register long __r2 __asm__( "r2" ) = (long) arg3;			\
    register long __r3 __asm__( "r3" ) = (long) arg4;			\
    register long __r4 __asm__( "r4" ) = (long) arg5;			\
    register long __res_r0 __asm__( "r0" );				\
    long __res;					       			\
    __asm__ __volatile__ (						\
      CRI_syscall( nr )				       			\
  	: "=r" (__res_r0)						\
  	: CRI_SYS_REG_LIST( "0" (__r0), "r" (__r1), "r" (__r2),		\
  		  	    "r" (__r3), "r" (__r4) )			\
  	: "memory" );							\
    __res = __res_r0;							\
    cri_syscall_return( type, __res, errno_p );	       			\
  }

#define cri_syscall6(type,name,nr,type1,type2,type3,type4,type5,type6)	\
  type name(type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5, type6 arg6, int *errno_p) { \
    CRI_SYS_REG( nr )							\
    register long __r0 __asm__( "r0" ) = (long) arg1;			\
    register long __r1 __asm__( "r1" ) = (long) arg2;			\
    register long __r2 __asm__( "r2" ) = (long) arg3;			\
    register long __r3 __asm__( "r3" ) = (long) arg4;			\
    register long __r4 __asm__( "r4" ) = (long) arg5;			\
    register long __r5 __asm__( "r5" ) = (long) arg6;			\
    register long __res_r0 __asm__( "r0" );				\
    long __res;					       			\
    __asm__ __volatile__ (						\
      CRI_syscall( nr )				       			\
  	: "=r" (__res_r0)						\
  	: CRI_SYS_REG_LIST( "0" (__r0), "r" (__r1), "r" (__r2),		\
  			    "r" (__r3), "r" (__r4), "r" (__r5) )	\
  	: "memory" );							\
    __res = __res_r0;							\
    cri_syscall_return( type, __res,errno_p );				\
  }

// signal return trampoline, from glibc-2.3.2
// Note: some of the below code is adapted from the GNU C library
// Copyright (C) 1997, 1998 Free Software Foundation, Inc.
#include <asm/unistd.h>
#ifndef __NR_rt_sigreturn
#  error "no value for __NR_rt_sigreturn"
#endif

#if defined(__thumb__)
#  define CRI_align                    ".align 2"
#elif defined(__ARM_EABI__)
#  define CRI_align                    ".align 8"
#else // standard ARM
#  define CRI_align                    ".align 4"
#endif

#if defined(__thumb__) || defined(__ARM_EABI__)
#define CRI_SA_RESTORER3(name, syscall) 		\
  __asm__ ( 						\
    ".text"				"\n\t"		\
    ".type " #name "," "%function"	"\n\t"		\
    CRI_align				"\n"		\
    #name ":" 				"\n\t" 		\
    "mov" "\t" "r7, #" #syscall		"\n\t"  \
    CRI_syscall( syscall )		"\n\t"		\
  );							\
  static void cri_sa_restorer( void ) __asm__( #name );
#else // neither __thumb__ nor __ARM_EABI__ are defined
#define CRI_SA_RESTORER3(name, syscall) 		\
  __asm__ ( 						\
    ".text"				"\n\t"		\
    ".type " #name "," "%function"	"\n\t"		\
    CRI_align				"\n"		\
    #name ":" 				"\n\t" 		\
    CRI_syscall( syscall )		"\n"		\
  );							\
  static void cri_sa_restorer( void ) __asm__( #name );
#endif // end if defined __thumb__ or __ARM_EABI__
#define CRI_SA_RESTORER2(name, syscall) CRI_SA_RESTORER3(name, syscall)
#define CRI_SA_RESTORER CRI_SA_RESTORER2(__restore_rt, __NR_rt_sigreturn)
                                                                                                              
#ifndef SA_RESTORER
#  define SA_RESTORER 0x04000000
#endif

/* Also needed w/ RESTORER: ensures we return via the proper restorer (see bug 2003) */
#define cri_restorer_jump(__signr, __siginfo, __context, __restorer) \
    __asm__ __volatile__ (  \
	"mov\tsp, %0\n\t"   \
	"mov\tpc, %1"       \
	: : "r" ((unsigned long)(__siginfo)), "r" (&(__restorer)));

#endif // ifdef _CR_ARCH_H
