/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_arch.h,v 1.11 2007/08/21 00:12:22 phargrov Exp $
 */

#ifndef _CR_ARCH_H
#define _CR_ARCH_H	1

// Catch-all file for misc. arch-specific bits

#ifdef PIC
  #define CRI_SYSCALL_LD_ARG1 \
	"\tmovl %2,%%ebx\n"
  #define CRI_SYSCALL_PRE \
	"\tpushl %%ebx\n"				\
	CRI_SYSCALL_LD_ARG1
  #define CRI_SYSCALL_POST \
	"\tpopl %%ebx\n"
  #define CRI_SYSCALL_ARG1 "r"
#else
  #define CRI_SYSCALL_LD_ARG1
  #define CRI_SYSCALL_PRE
  #define CRI_SYSCALL_POST
  #define CRI_SYSCALL_ARG1 "b"
#endif


#define cri_syscall_cleanup(res,errno_p) \
    __asm__ volatile ("" ::: "memory", "cc", "cx", "dx");\
    if ((unsigned long)res >= (unsigned long)(-4096)) {	\
	if (errno_p != NULL) { *errno_p = -res; }	\
	res = -1;					\
    }

// XXX: does syscall5 work w/ PIC?

#define cri_syscall5(type,name,nr,type1,type2,type3,type4,type5)\
  type name(type1 arg1,type2 arg2,type3 arg3,type4 arg4,type5 arg5,int *errno_p) {\
      long _res;					\
      __asm__ volatile (				\
	CRI_SYSCALL_PRE					\
	"\tint $0x80\n"					\
	CRI_SYSCALL_POST				\
        : "=a" (_res)					\
        : "0" (nr),					\
	  CRI_SYSCALL_ARG1 ((long)(arg1)),		\
	  "c" ((long)(arg2)),				\
	  "d" ((long)(arg3)),				\
	  "S" ((long)(arg4)),				\
	  "D" ((long)(arg5)));				\
      cri_syscall_cleanup(_res, errno_p);		\
      return (type)_res;				\
  }

#define cri_syscall4(type,name,nr,type1,type2,type3,type4)\
  type name(type1 arg1,type2 arg2,type3 arg3,type4 arg4,int *errno_p) {\
      long _res;					\
      __asm__ volatile (				\
	CRI_SYSCALL_PRE					\
	"\tint $0x80\n"					\
	CRI_SYSCALL_POST				\
        : "=a" (_res)					\
        : "0" (nr),					\
	  CRI_SYSCALL_ARG1 ((long)(arg1)),		\
	  "c" ((long)(arg2)),				\
	  "d" ((long)(arg3)),				\
	  "S" ((long)(arg4)));				\
      cri_syscall_cleanup(_res, errno_p);		\
      return (type)_res;				\
  }

#define cri_syscall3(type,name,nr,type1,type2,type3)\
  type name(type1 arg1,type2 arg2,type3 arg3,int *errno_p) {\
      long _res;					\
      __asm__ volatile (				\
	CRI_SYSCALL_PRE					\
	"\tint $0x80\n"					\
	CRI_SYSCALL_POST				\
        : "=a" (_res)					\
        : "0" (nr),					\
	  CRI_SYSCALL_ARG1 ((long)(arg1)),		\
	  "c" ((long)(arg2)),				\
	  "d" ((long)(arg3)));				\
      cri_syscall_cleanup(_res, errno_p);		\
      return (type)_res;				\
  }

#define cri_syscall2(type,name,nr,type1,type2)\
  type name(type1 arg1,type2 arg2,int *errno_p) {\
      long _res;					\
      __asm__ volatile (				\
	CRI_SYSCALL_PRE					\
	"\tint $0x80\n"					\
	CRI_SYSCALL_POST				\
        : "=a" (_res)					\
        : "0" (nr),					\
	  CRI_SYSCALL_ARG1 ((long)(arg1)),		\
	  "c" ((long)(arg2)));				\
      cri_syscall_cleanup(_res, errno_p);		\
      return (type)_res;				\
  }

#define cri_syscall1(type,name,nr,type1)\
  type name(type1 arg1,int *errno_p) {\
      long _res;					\
      __asm__ volatile (				\
	CRI_SYSCALL_PRE					\
	"\tint $0x80\n"					\
	CRI_SYSCALL_POST				\
        : "=a" (_res)					\
        : "0" (nr),					\
	  CRI_SYSCALL_ARG1 ((long)(arg1)));		\
      cri_syscall_cleanup(_res, errno_p);		\
      return (type)_res;				\
  }

#define cri_syscall0(type,name,nr)\
  type name(int *errno_p) {\
      long _res;					\
      __asm__ volatile (				\
	"\tint $0x80\n"					\
        : "=a" (_res)					\
        : "0" (nr));					\
      cri_syscall_cleanup(_res, errno_p);		\
      return (type)_res;				\
  }

/* On an x86_64 the registers ebx and ebp are callee-saved.
 * Currently we lack a way to know this is a 64-bit CPU.
 * So, we do this unconditionally.
 */
#define cri_syscall3X(type,name,nr,type1,type2,type3)\
  type name(type1 arg1,type2 arg2,type3 arg3,int *errno_p) {\
      long _res;					\
      __asm__ volatile (				\
	"\tpushl %%ebp\n"				\
	"\tpushl %%ebx\n"				\
	CRI_SYSCALL_LD_ARG1				\
	"\tint $0x80\n"					\
	"\tpopl %%ebx\n"				\
	"\tpopl %%ebp\n"				\
        : "=a" (_res)					\
        : "0" (nr),					\
	  CRI_SYSCALL_ARG1 ((long)(arg1)),		\
	  "c" ((long)(arg2)),				\
	  "d" ((long)(arg3)));				\
      cri_syscall_cleanup(_res, errno_p);		\
      return (type)_res;				\
  }

/* signal return trampoline, from glibc-2.3.2 */
#include <asm/unistd.h>
#ifndef __NR_rt_sigreturn
  #error "no value for __NR_rt_sigreturn"
#endif
#define CRI_SA_RESTORER3(name, syscall) \
  __asm__ (                             \
      ".text\n"                         \
      ".align 16\n"                     \
      #name ":\n"                       \
      "    movl $" #syscall ", %eax\n"  \
      "    int $0x80\n"                 \
  );                                    \
  static void cri_sa_restorer(void) __asm__(#name);
#define CRI_SA_RESTORER2(name, syscall) CRI_SA_RESTORER3(name, syscall)
#define CRI_SA_RESTORER CRI_SA_RESTORER2(__restore_rt, __NR_rt_sigreturn)
                                                                                                              
#ifndef SA_RESTORER
  #define SA_RESTORER 0x04000000
#endif

/* Also needed w/ RESTORER: ensures we return via the proper restorer (see bug 2003) */
#define cri_restorer_jump(__signr, __siginfo, __context, __restorer) \
    __asm__ __volatile__ (				\
        "mov    %0,%%esp\n\t"				\
        "jmp    *%1"					\
        : : "r" (&(__signr)), "a" (&(__restorer)))

#endif
